/*
sio4b_sync_multichannel_sample.c:

=========================================================================
Version:	Date:			Purpose:				Initials:
-------------------------------------------------------------------------
1.0			10-29-2004		Initial code			jdp

=========================================================================

This test demonstrates a channel to channel data test using the PLXSW-SIO4B API
with the SIO4B/BX-SYNC version of the SIO4 family of cards.  The -SYNC variation
is a protocol independent, simple serial->parallel card.  This test is designed
ONLY for the -SYNC variation of the SIO4B/BX.  Please check the part number
on your card to be sure that this test will be supported.

Channels are connected to one another via a GSC provided cable (such as
cableX-sio4B-std2-db25X) and DB25 connectors are connected via "gender
changers".

This test requires a 4 channel cable terminated with DB25 connectors.
For this test, channels are connects as follows: 1->2, 3->4, using
"gender changers" to connect each pair of channels.
*/

#include <stdio.h>
#include <conio.h>
#include <windows.h>

#include "..\..\..\Include\GscApi.h"

//
//	Global Variables:
//
#define	BUFFER_SIZE		16
static double fTestVersion = 1.0;

//
//	Function Declarations:
//
void DisplayErrorMessage( int errorCode);
void ClearScreen(void);
int	loopbackDataTest(int boardNumber, int txChannel, int rxChannel);

//
//	main():
//
int main(void)
{
	int status;
	int libVersion, driverVersion, fpgaVersion;

	// init tx and rx channels
	int iTxChannel;
	int iRxChannel;
	int iboardNumber = 1;

	printf("\n SIO4B-SYNC Channel to Channel Test version %.2f", fTestVersion);

	//
	// open board number iboardNumber
	//
	status = GscSio4Open(iboardNumber);
	if(status)
		DisplayErrorMessage(status);

	//
	//	Display Versions: 
	//
	status = GscSio4GetVersions(iboardNumber, &libVersion, &driverVersion, &fpgaVersion);
	if(status)
		DisplayErrorMessage(status);
	else
	{
		printf("\n ----------------------------------------------");
		printf("\n API Version:\t\t0x%08lX", libVersion);
		printf("\n Driver Version:\t0x%08lX", driverVersion);
		printf("\n Firmware Version:\t0x%08lX", fpgaVersion);
		printf("\n cpTxBuffer Size:\t%d bytes", BUFFER_SIZE);
		printf("\n ----------------------------------------------");
	}

	printf("\n This test requires a 4 channel cable terminated with DB25 connectors.");
	printf("\n For this test, channels are connects as follows: 1->3, 2->4, using \n \"gender changers\" to connect each pair of channels.");
	printf("\n Press a key to run test...");
	getch();

	//
	//	Run Tests:
	//
	iTxChannel = 1;
	iRxChannel = 3;
	if(status = loopbackDataTest(iboardNumber, iTxChannel, iRxChannel))
		DisplayErrorMessage(status);

	iTxChannel = 3;
	iRxChannel = 1;
	if(status = loopbackDataTest(iboardNumber, iTxChannel, iRxChannel))
		DisplayErrorMessage(status);

	iTxChannel = 2;
	iRxChannel = 4;
	if(status = loopbackDataTest(iboardNumber, iTxChannel, iRxChannel))
		DisplayErrorMessage(status);

	iTxChannel = 4;
	iRxChannel = 2;
	if(status = loopbackDataTest(iboardNumber, iTxChannel, iRxChannel))
		DisplayErrorMessage(status);


	//
	// close handle to SIO4 Board 
	//
	status = GscSio4Close(iboardNumber);
	if(status)
		DisplayErrorMessage(status);

	printf("\n Press any key to exit...");
	getch();

	printf("\n");
	return 0;
}


//
//	DisplayErrorMessage
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long.
//
void DisplayErrorMessage( int errorCode)
{
	// Buffer to hold the error message
	char errorMessage[80];

	// Call the API to interpret the message
	GscSio4GetErrorString( errorCode, errorMessage);

	// Display the error
	printf("\n\n\tError Code: %d, %s", errorCode, errorMessage);
	printf("\n\tPress any key to continue\n\n");

	// Wait for a key press
	getch();
}


//
//	Clears a console screen
//
void ClearScreen(void)
{
  HANDLE MainConsole;
  COORD coordScreen = {0,0};
  DWORD cCharsWritten;
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  DWORD dwConSize;

  // Get a HANDLE to the Console Window
  MainConsole = GetStdHandle(STD_OUTPUT_HANDLE);

  // Get Console Information
  GetConsoleScreenBufferInfo(MainConsole, &csbi );

  // Figure Console Size
  dwConSize = csbi.dwSize.X * csbi.dwSize.Y;

  // Fill Console with spaces
  FillConsoleOutputCharacter(MainConsole, (TCHAR) ' ', dwConSize, coordScreen, &cCharsWritten );

  // Get Console Information
  GetConsoleScreenBufferInfo(MainConsole, &csbi );

  // Fill Console with Current Attributes
  FillConsoleOutputAttribute(MainConsole, csbi.wAttributes, dwConSize, coordScreen, &cCharsWritten );

  // Send the Cursor Home
  SetConsoleCursorPosition(MainConsole, coordScreen );
  return;
}

int	loopbackDataTest(int iboardNumber, int iTxChannel, int iRxChannel)
{
	int status = 0, result;
	int rxid, txid, iCounter, iErrors, bytesXfered;
	unsigned char *cpRxBuffer, *cpTxBuffer;

	cpTxBuffer = malloc(BUFFER_SIZE);
	cpRxBuffer = malloc(BUFFER_SIZE);

	// initialize data buffers (tx and rx)
	for(iCounter = 0; iCounter < BUFFER_SIZE; iCounter++)
		*(cpTxBuffer + iCounter) = (char)iCounter;

	memset(cpRxBuffer, 0x0, BUFFER_SIZE);

	//
	//	perform channel resets to put everything in a known state:
	//
	status = GscSio4ChannelReset(iboardNumber, iTxChannel);
	if(status) return(status);

	status = GscSio4ChannelReset(iboardNumber, iRxChannel);
	if(status) return(status);

	//
	//	set channel modes to GSC_MODE_SYNC_ENV, 1 Mbit/sec - see gscapi.h for options
	//
	status = GscSio4ChannelSetMode(iboardNumber, iTxChannel, GSC_MODE_SYNC_ENV, 1000000);
	if(status) return(status);

	status = GscSio4ChannelSetMode(iboardNumber, iRxChannel, GSC_MODE_SYNC_ENV, 1000000);
	if(status) return(status);
	
	//
	//	set message size in bits
	//
	status = GscSio4ChannelSetDataSize(iboardNumber, iTxChannel, 16);
	if(status) return(status);

	status = GscSio4ChannelSetDataSize(iboardNumber, iRxChannel, 16);
	if(status) return(status);

	//
	//	set gap size in clock cycles
	//
	status = GscSio4ChannelSetGapSize(iboardNumber, iTxChannel, 4);
	if(status) return(status);

	status = GscSio4ChannelSetGapSize(iboardNumber, iRxChannel, 4);
	if(status) return(status);

	// API defaults channels to DTE with the following settings:
	// Transmit Data on cable upper
	// Receive Data on cable lower
	//
	// with two channels connected via a gender changer, one channel must have
	// this reversed in order to properly swap data between channels
	// Set Rx channel DCE
	status = GscSio4ChannelSetDteDce(iboardNumber, iRxChannel, GSC_PIN_DCE);
	if(status) return(status);

	printf("\n Transmit Channel -> %d  Receive Channel -> %d", iTxChannel, iRxChannel);

	
	//-------------------------------------------------------
	//      FIFO reset workaround (for API v1.2.9 - 1.2.C)
	//-------------------------------------------------------
	//      reset FIFOs:
	GscSio4LocalRegisterRead( iboardNumber,
			(CONTROL_STATUS_BASE_REG + ((iTxChannel - 1) << 4)), &result);
	GscSio4LocalRegisterWrite( iboardNumber,
			(CONTROL_STATUS_BASE_REG + ((iTxChannel - 1) << 4)), (result | 0x3));
	
	//      reset FIFOs:
	GscSio4LocalRegisterRead( iboardNumber,
			(CONTROL_STATUS_BASE_REG + ((iRxChannel - 1) << 4)), &result);
	GscSio4LocalRegisterWrite( iboardNumber,
			(CONTROL_STATUS_BASE_REG + ((iRxChannel - 1) << 4)), (result | 0x3));
	//-------------------------------------------------------
	//      /FIFO reset workaround
	//-------------------------------------------------------
 
	
	// begin data xfer...
		iErrors = 0;

		// queue data write to Tx channel
		status = GscSio4ChannelTransmitData(iboardNumber, iTxChannel, cpTxBuffer, BUFFER_SIZE, &txid);
		if(status) return(status);

/*
		// wait for up to 5 seconds for data write to complete
		status = GscSio4ChannelWaitForTransfer(iboardNumber, iTxChannel, 5000, txid, &bytesXfered);
		if(status) return(status);
		else if(bytesXfered != -1)
			printf("\n Bytes Transmitted: %d", bytesXfered);
		else
			printf("\n Bytes Transmitted: %d", BUFFER_SIZE);
*/
		// allow serial data to finish xfer to Rx
		Sleep(1);

		// queue data read from Rx channel
		printf("\n Receiving Data...");
		status = GscSio4ChannelReceiveData(iboardNumber, iRxChannel, cpRxBuffer, BUFFER_SIZE, &rxid);
		if(status) return(status);
		
		// wait for up to 5 seconds for data read from Rx channel to complete
		status = GscSio4ChannelWaitForTransfer(iboardNumber, iRxChannel, 5000, rxid, &bytesXfered);
		if(status) return(status);
		else if(bytesXfered != -1)
			printf("\n Bytes Received: %d", bytesXfered);
		else
			printf("\n Bytes Received: %d", BUFFER_SIZE);


		//
		//	Check data validity
		//
		for(iCounter = 0; iCounter < BUFFER_SIZE; iCounter++)
		{
			if(*(cpRxBuffer + iCounter) != *(cpTxBuffer + iCounter))
				iErrors++;
			if(iErrors > 0 && iErrors < 10)
				printf("\n Data error at offset 0x%02X: Wrote %02X - Read %02X", iCounter, *(cpTxBuffer + iCounter), *(cpRxBuffer + iCounter));
		}

		printf("\n *** Channel to Channel Test:");

		if(iErrors)
			printf(" FAILED - %d Errors ***\n", iErrors);
		else
			printf(" Passed ***\n");

		//
		//	Clear out Rx Buffer
		//
		memset(cpRxBuffer, 0x0, BUFFER_SIZE);

		//
		// free memory
		//
		free(cpTxBuffer);
		free(cpRxBuffer);


		if(status)
			return (status);
		else
			return 0;
}

