/*
SIO4b_sync_sample.c:

=========================================================================
Version:	Date:			Purpose:					Initials:
-------------------------------------------------------------------------
1.0			01-20-2005		Initial code				jdp
1.1			04-05-2006		Added FIFO reset workaround	jdp

=========================================================================

This test demonstrates a simple loopback test of the SIO4B-SYNC/SIO4BX-SYNC
versions of the SIO4 family of cards.  The -SYNC variation is a protocol 
independent, simple serial->parallel card.  This test performs an "external" 
loopback test on the SIO4B-SYNC.  The data/clock are looped back at the 
transceivers, so no cable should be connected during the test.


NOTE: This test is designed ONLY for the -SYNC variation of the SIO4B/BX.  
Please check the part number on your card to be sure that this test will 
be supported.

No cable should be connected to the card for this test
*/

#include <stdio.h>
#include <conio.h>
#include <windows.h>

#include "..\..\..\Include\GscApi.h"

//
//	Global Variables:
//
#define	BUFFER_SIZE		128
static double fTestVersion = 1.0;

//
//	Function Declarations:
//
void DisplayErrorMessage( int errorCode);
void ClearScreen(void);
int	loopbackDataTest(int iboardNumber, int iChannel, int iLoops);

//
//	main():
//
int main(void)
{
	int status;
	int libVersion, driverVersion, fpgaVersion;

	int iboardNumber = 1, iChannel = 1, iLoop;

	printf("\n SIO4B-SYNC Simple Loopback Test version %.2f", fTestVersion);

	//
	// open board number iboardNumber
	//
	status = GscSio4Open(iboardNumber);
	if(status)
		DisplayErrorMessage(status);

	//
	//	Display Versions: 
	//
	status = GscSio4GetVersions(iboardNumber, &libVersion, &driverVersion, &fpgaVersion);
	if(status)
		DisplayErrorMessage(status);
	else
	{
		printf("\n ----------------------------------------------");
		printf("\n API Version:\t\t0x%08lX", libVersion);
		printf("\n Driver Version:\t0x%08lX", driverVersion);
		printf("\n Firmware Version:\t0x%08lX", fpgaVersion);
		printf("\n cpTxBuffer Size:\t%d bytes", BUFFER_SIZE);
		printf("\n ----------------------------------------------");
	}

	printf("\n Please be sure to disconnect all cables for this test.\n Press any key to run test...");
	getch();

	// check to be sure this is a -SYNC version
	if( (fpgaVersion & 0xFF00) < 0x0300)
	{
		printf("\n This board does not appear to be a -SYNC version.");
		printf("\n Press any key to exit test...");
		getch();
		return 1;
	
	}


	for(iLoop = 1; iLoop < 5; iLoop++)
	{
		// perform test(s) on channels 1 - 4
		if(status = loopbackDataTest(iboardNumber, iLoop, 0x1))
			DisplayErrorMessage(status);
	}

		
	printf("\n Press any key to exit...");
	getch();

	//
	// close handle to SIO4 Board 
	//
	status = GscSio4Close(iboardNumber);
	if(status)
		DisplayErrorMessage(status);

	printf("\n");
	return 0;
}


//
//	DisplayErrorMessage
//
//  This routine calls the API to get the text that
// corresponds to a numeric error code. The routine is
// guaranteed to return a valid string and the string
// will always be less than 80 characters long.
//
void DisplayErrorMessage( int errorCode)
{
	// Buffer to hold the error message
	char errorMessage[80];

	// Call the API to interpret the message
	GscSio4GetErrorString( errorCode, errorMessage);

	// Display the error
	printf("\n\n\tError Code: %d, %s", errorCode, errorMessage);
	printf("\n\tPress any key to continue\n\n");

	// Wait for a key press
	getch();
}


//
//	Clears a console screen
//
void ClearScreen(void)
{
  HANDLE MainConsole;
  COORD coordScreen = {0,0};
  DWORD cCharsWritten;
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  DWORD dwConSize;

  // Get a HANDLE to the Console Window
  MainConsole = GetStdHandle(STD_OUTPUT_HANDLE);

  // Get Console Information
  GetConsoleScreenBufferInfo(MainConsole, &csbi );

  // Figure Console Size
  dwConSize = csbi.dwSize.X * csbi.dwSize.Y;

  // Fill Console with spaces
  FillConsoleOutputCharacter(MainConsole, (TCHAR) ' ', dwConSize, coordScreen, &cCharsWritten );

  // Get Console Information
  GetConsoleScreenBufferInfo(MainConsole, &csbi );

  // Fill Console with Current Attributes
  FillConsoleOutputAttribute(MainConsole, csbi.wAttributes, dwConSize, coordScreen, &cCharsWritten );

  // Send the Cursor Home
  SetConsoleCursorPosition(MainConsole, coordScreen );
  return;
}

//---------------------------------------------------------------------
//
//	This test performs an "external" loopback test on the SIO4B-SYNC
//	The data/clock are looped back at the transceivers, so no cable
//	should be connected during the test
//
//---------------------------------------------------------------------
int	loopbackDataTest(int iboardNumber, int iChannel, int iLoops)
{
	int status = 0, result;
	int rxid, txid, iCounter, iErrors, bytesXfered;
	unsigned char *cpRxBuffer, *cpTxBuffer;

	cpTxBuffer = malloc(BUFFER_SIZE);
	cpRxBuffer = malloc(BUFFER_SIZE);

	// initialize data buffers (tx and rx)
	for(iCounter = 0; iCounter < BUFFER_SIZE; iCounter++)
		*(cpTxBuffer + iCounter) = (char)iCounter;

	memset(cpRxBuffer, 0x0, BUFFER_SIZE);


	//
	//	perform channel resets to put everything in a known state:
	//
	status = GscSio4ChannelReset(iboardNumber, iChannel);
	if(status) return(status);

	//
	//	set channel modes to GSC_MODE_SYNC_ENV, 1 Mbit/sec - see gscapi.h for options
	//
	status = GscSio4ChannelSetMode(iboardNumber, iChannel, GSC_MODE_SYNC_ENV, 1000000);
	if(status) return(status);
	
	//
	//	set message size in bits
	//
	status = GscSio4ChannelSetDataSize(iboardNumber, iChannel, 8);
	if(status) return(status);

	//
	//	set gap size in clock cycles
	//
	status = GscSio4ChannelSetGapSize(iboardNumber, iChannel, 2);
	if(status) return(status);

	//
	//	set external loopback
	//
	status = GscSio4ChannelSetLoopBack(iboardNumber, iChannel, GSC_LOOP_EXTERNAL);
	if(status) return(status);


	//-------------------------------------------------------
	//      FIFO reset workaround (for API v1.2.9 - 1.2.C)
	//-------------------------------------------------------
	//      reset FIFOs:
	GscSio4LocalRegisterRead( iboardNumber,
			(CONTROL_STATUS_BASE_REG + ((iChannel - 1) << 4)), &result);
	GscSio4LocalRegisterWrite( iboardNumber,
			(CONTROL_STATUS_BASE_REG + ((iChannel - 1) << 4)), (result | 0x3));
	//-------------------------------------------------------
	//      /FIFO reset workaround
	//-------------------------------------------------------
 

	//
	// convenience loop
	//
	for(;;)
	{
		iErrors = 0;
		iLoops--;

		printf("\n Testing channel: %d", iChannel);

		// queue data write to Tx channel
		status = GscSio4ChannelTransmitData(iboardNumber, iChannel, cpTxBuffer, BUFFER_SIZE, &txid);
		if(status) return(status);

		// wait for up to 5 seconds for data write to complete
		status = GscSio4ChannelWaitForTransfer(iboardNumber, iChannel, 200, txid, &bytesXfered);
		if(status) return(status);
		else if(bytesXfered != -1)
			printf("\n Bytes Transmitted: %d", bytesXfered);
		else
			printf("\n Bytes Transmitted: %d", BUFFER_SIZE);

		// allow serial data to finish xfer to Rx
		Sleep(1);


		//
		//	Clear out Rx Buffer
		//
		memset(cpRxBuffer, 0x0, BUFFER_SIZE);
	
		//
		// queue data read from Rx channel
		//
		status = GscSio4ChannelReceiveData(iboardNumber, iChannel, cpRxBuffer, BUFFER_SIZE, &rxid);
		if(status) return(status);
		
		//
		// wait for up to 5 seconds for data read from Rx channel to complete
		//
		status = GscSio4ChannelWaitForTransfer(iboardNumber, iChannel, 200, rxid, &bytesXfered);
		if(status) return(status);
		else if(bytesXfered != -1)
			printf("\n Bytes Received: %d", bytesXfered);
		else
			printf("\n Bytes Received: %d", BUFFER_SIZE);


		//
		//	Check data validity
		//
		for(iCounter = 0; iCounter < BUFFER_SIZE; iCounter++)
		{
			if(*(cpRxBuffer + iCounter) != *(cpTxBuffer + iCounter))
				iErrors++;
			if(iErrors > 0 && iErrors < 10)
				printf("\n Data error at offset 0x%02X: Wrote %02X - Read %02X", iCounter, *(cpTxBuffer + iCounter), *(cpRxBuffer + iCounter));
		}

		if(iErrors)
		{
			printf("\n *** Loopback Data Test Channel %d FAILED - %d Errors ***\n", iChannel, iErrors);
			if(toupper(getch()) == 'X')
				break;
		}
		else
			printf("\n *** Loopback Data Test Channel %d Passed ***\n", iChannel);

		
		Sleep(200);
		
		if(kbhit())
			break;
		
		// when iLoops gets to 0, break
		if(!iLoops)
			break;

//		break;
	} // end for()

	//
	// free memory
	//
	free(cpTxBuffer);
	free(cpRxBuffer);


	if(status)
		return (status);
	else
		return 0;
}

